@@ -97,6 +97,8 @@ githubactionsvariable = ""
 azurefilter = ""
 replaceretry = "false"
 webdriverio = "false"
+percentage = ""
+endspecificrun = ""
 
 def find(name):
     currentdir = os.getcwd() # using current dir, could change this to work with full computer search
@@ -463,6 +465,9 @@ def get_tests(testpriority):
         params["target_branch"] = branch
         params["from_commit"] = fromcommit
 
+    if percentage.isnumeric():
+        params["percentage"] = percentage
+    
     print(params)
 
     if proxy == "":
@@ -504,7 +509,6 @@ def get_tests(testpriority):
 def get_and_run_tests(type):
     testset = get_tests(type)
     count=0
-    
     tests = ""
     try:    
         for element in testset:
@@ -668,29 +672,49 @@ def call_import(filepath):
             openfile.write(filedata)
 
     sizeOfFile = os.path.getsize(filepath)
-    if importtype == "trx" and sizeOfFile > 1000000:
-        ET.register_namespace("", "http://microsoft.com/schemas/VisualStudio/TeamTest/2010")
-        tree = ET.ElementTree()
-        tree.parse(filepath)
-        root = tree.getroot()
-        ns = re.match(r'{.*}', root.tag).group(0)
-        for test in root.findall(f"{ns}Results"):
-            for testresult in test.findall(f"{ns}UnitTestResult"):
+    #if importtype == "trx" and sizeOfFile > 1000000:
+    #    ET.register_namespace("", "http://microsoft.com/schemas/VisualStudio/TeamTest/2010")
+    #    tree = ET.ElementTree()
+    #    tree.parse(filepath)
+    #    root = tree.getroot()
+    #    ns = re.match(r'{.*}', root.tag).group(0)
+    #    for test in root.findall(f"{ns}Results"):
+    #        for testresult in test.findall(f"{ns}UnitTestResult"):
                 #testName = (testresult.get("testName"))
                 #sep = ','
                 #testName = testName.split(sep, 1)[0]
                 #sep = ' '
                 #testName = testName.split(sep, 1)[0]
                 #testresult.set("testName", testName)
-                for output in testresult.findall(f"{ns}Output"):
-                    for stderr in output.findall(f"{ns}StdOut"):
-                        stderr.text = stderr.text[:500]
-                    for stderr in output.findall(f"{ns}StdErr"):
-                        stderr.text = stderr.text[:500]
-                    for ErrorInfo in output.findall(f"{ns}ErrorInfo"):
-                        for stacktrace in ErrorInfo.findall(f"{ns}StackTrace"):
-                            stacktrace.text = stacktrace.text[:500]
-        tree.write(filepath)
+    #            for output in testresult.findall(f"{ns}Output"):
+    #                for stderr in output.findall(f"{ns}StdOut"):
+    #                    stderr.text = stderr.text[:500]
+    #                for stderr in output.findall(f"{ns}StdErr"):
+    #                    stderr.text = stderr.text[:500]
+    #                for ErrorInfo in output.findall(f"{ns}ErrorInfo"):
+    #                    for stacktrace in ErrorInfo.findall(f"{ns}StackTrace"):
+    #                        stacktrace.text = stacktrace.text[:500]
+    #    tree.write(filepath)
+
+    if importtype == "trx" and sizeOfFile > 1000000:
+        ET.register_namespace("", "http://microsoft.com/schemas/VisualStudio/TeamTest/2010")
+        tree = ET.ElementTree()
+        with open(filepath, 'r', errors='replace',encoding="utf8") as f:
+            root = ET.fromstring(f.read())
+            tree = ET.ElementTree()
+            ns = re.match(r'{.*}', root.tag).group(0)
+            for test in root.findall(f"{ns}Results"):
+                for testresult in test.findall(f"{ns}UnitTestResult"):
+                    for output in testresult.findall(f"{ns}Output"):
+                        for stderr in output.findall(f"{ns}StdOut"):
+                            stderr.text = stderr.text[:500]
+                        for stderr in output.findall(f"{ns}StdErr"):
+                            stderr.text = stderr.text[:500]
+                        for ErrorInfo in output.findall(f"{ns}ErrorInfo"):
+                            for stacktrace in ErrorInfo.findall(f"{ns}StackTrace"):
+                                stacktrace.text = stacktrace.text[:500]
+            tree._setroot(root)
+            tree.write(filepath)
 
     if webdriverio == "true":
         tree = ElementTree()
@@ -766,7 +790,7 @@ def runtestswithappsurify(*args):
     global commit, scriptlocation, branch, runfrequency, fromcommit, repository, scriptlocation, generatefile, template, addtestsuitename, addclassname, runtemplate, testsuitesnameseparator
     global testtemplate, classnameseparator, testseparatorend, testtemplatearg1, testtemplatearg2, testtemplatearg3, testtemplatearg4, startrunpostfix, endrunprefix
     global endrunpostfix, executetests, encodetests, testsuiteencoded, projectencoded, testsrun, trainer, azure_variable, pipeoutput, recursive, bitrise, executioncommand, githubactionsvariable, printcommand
-    global azurefilter, replaceretry, webdriverio
+    global azurefilter, replaceretry, webdriverio, percentage, endspecificrun
 
     tests=""
     testsrun=""
@@ -837,6 +861,8 @@ def runtestswithappsurify(*args):
     azurefilter = ""
     replaceretry = "false"
     webdriverio = "false"
+    percentage = ""
+    endspecificrun=""
     #--testsuitesnameseparator and classnameseparator need to be encoded i.e. # is %23
 
 
@@ -1128,6 +1154,7 @@ def runtestswithappsurify(*args):
         postfixtest="'"
         prefixtest="'"
         startrunall="cypress run --reporter junit --reporter-options mochaFile=result.xml"
+        endspecificrun = " grep=\""
 
     #mstest
     #/Tests:TestMethod1,testMethod2
@@ -1141,6 +1168,7 @@ def runtestswithappsurify(*args):
         startrunall="mstest /resultsfile:'" + testtemplatearg1 + "' /testcontainer:'" + testtemplatearg2 + "'"
         report=testtemplatearg1
         importtype="trx"
+        
 
 
     #vstest
@@ -1385,6 +1413,13 @@ def runtestswithappsurify(*args):
                 printcommand = argv[k+1]
             if argv[k] == "--azurefilter":
                 azurefilter = argv[k+1]
+            if argv[k] == "--percentage":
+                percentage = argv[k+1]
+            if argv[k] == "--runcommand":
+                startrunall = argv[k+1]
+                startrunspecific = argv[k+1] + endspecificrun
+                print("fall back command = "+ startrunall)
+                print("prioritized run = "+ startrunspecific)
             if argv[k] == "--help":
                 echo("please see url for more details on this script and how to execute your tests with appsurify - https://github.com/Appsurify/AppsurifyCI")
 
@@ -1468,7 +1503,7 @@ def runtestswithappsurify(*args):
         echo("startrunall needs to be set in order to execute tests")
         exit(1)
     if startrunspecific == "" and teststorun == "all" and rerun == "true":
-        echo("startrunspecific needs to be set in order to rerun tests, either set rerun to false or set startrunaspecific")
+        echo("startrunspecific needs to be set in order to rerun tests, either set rerun to false or set startrunspecific")
         exit(1)
 
     #if [[ $teststorun == "" ]] ; then echo "no teststorun specified" ; exit 1 ; fi
@@ -1498,21 +1533,27 @@ def runtestswithappsurify(*args):
 
     testtypes=[]
 
-    if "high" in teststorun:
-        testtypes.append(1)
-    if "medium" in teststorun:
-        testtypes.append(2)
-    if "low" in teststorun:
-        testtypes.append(3)
-    if "unassigned" in teststorun:
-        testtypes.append(4)
-    if "top20" in teststorun:
-        testtypes.append(8)
+    if percentage.isnumeric():
+        testtypes.append(9)
+    else:
+        if "high" in teststorun:
+            testtypes.append(1)
+        if "medium" in teststorun:
+            testtypes.append(2)
+        if "low" in teststorun:
+            testtypes.append(3)
+        if "unassigned" in teststorun:
+            testtypes.append(4)
+        if "top20" in teststorun:
+            testtypes.append(8)
 
     ####start loop
     for i in testtypes:
         #print(("testsrun1 = " + testsrun))
-        testsrun = get_and_run_tests(i) + testsrun
+        try:
+            testsrun = get_and_run_tests(i) + testsrun
+        except Exception as e:
+            print("Error running tests in set "+ i)
 
     #print("Tests to run")
     #print(testsrun)
